/*
 * @Author: zlf 
 * @Date: 2020-05-04 16:40:46 
 * @Last Modified by: zlf
 * @Last Modified time: 2020-05-04 17:21:35
 */
/// 创建一个以《json_annotation》第三方库解析的模型类/实体类
///
/// 主要目的是解放双手达到自动写出模型类/实体类的内容的目的
///
/// 创建的该类的内容会输出到控制台(然后进行复制到我们创建的类里面)
/// 记得查看控制台
class CreatClassModel {
  /**
   * 该函数提供打印出模型类/实体类的内容（字符串形式）形成一个模型类/实体类
   * 
   * [objc] --->需要解析成模型类/实体类的map对象
   * 
   * [className] --->模型类/实体类 的类名（名称）
   * 
   * [superClassName] --->模型类/实体类的父类类名（名称）默认继承 Object
   * 
   * 
   * 示例 :
   * 
   *     CreatClassModel.getPropertyFormMapWithCreatClassModel(jsonMap, "TestModel");
   * 
   *  [查看示例结果](https://user-gold-cdn.xitu.io/2020/5/4/171deea49c144e4d?w=1614&h=1362&f=png&s=211009)
   * 
   */
  static void getPropertyFormMapWithCreatClassModel(
      Map<String, dynamic> objc, String className,
      [String superClassName = "Object"]) {
    StringBuffer _classDes = StringBuffer();
    StringBuffer _propertyDes = StringBuffer();
    StringBuffer _thisDes = StringBuffer();
    StringBuffer _fromJsonDes = StringBuffer();
    StringBuffer _toJsonDes = StringBuffer();
    print("\n//————————————————————————从这里开始————————————————————————\n");
    _classDes
        .write("import 'package:json_annotation/json_annotation.dart'; \n\n");
    objc.forEach((k, v) {
      String _des = "";
      String _thisKey = "this.$k,";
      _toJsonDes.write("\t\t\'$k\': this.$k,\n");
      if (v is String) {
        _fromJsonDes.write("\t\tsrcJson[\'$k\'] as String,\n");
        _des = "\t@JsonKey(name: \'$k\')\n\tString $k;\n";
      } else if (v is int) {
        _fromJsonDes.write("\t\tsrcJson[\'$k\'] as int,\n");
        _des = "\t@JsonKey(name: \'$k\')\n\tint $k;\n";
      } else if (v is double) {
        _fromJsonDes.write("\t\tsrcJson[\'$k\'] as double,\n");
        _des = "\t@JsonKey(name: \'$k\')\n\tdouble $k;\n";
      } else if (v is bool) {
        _fromJsonDes.write("\t\tsrcJson[\'$k\'] as bool,\n");
        _des = "\t@JsonKey(name: \'$k\')\n\tbool $k;\n";
      } else if (v is List) {
        var firstObject = v.first;
        if (firstObject is String) {
          _fromJsonDes.write(
              "\t\t(srcJson[\'$k\'] as List)?.map((e) => e as String)?.toList(),\n");
          _des = "\t@JsonKey(name: \'$k\')\n\tList<String> $k;\n";
        } else if (firstObject is int) {
          _fromJsonDes.write(
              "\t\t(srcJson[\'$k\'] as List)?.map((e) => e as int)?.toList(),\n");
          _des = "\t@JsonKey(name: \'$k\')\n\tList<int> $k;\n";
        } else if (firstObject is double) {
          _fromJsonDes.write(
              "\t\t(srcJson[\'$k\'] as List)?.map((e) => e as double)?.toList(),\n");
          _des = "\t@JsonKey(name: \'$k\')\n\tList<int> $k;\n";
        } else if (firstObject is Map) {
          String objectName = "${k[0].toUpperCase()}${k.substring(1)}";
          _fromJsonDes.write(
              "\t\t(srcJson[\'$k\'] as List)?.map((e) => ${objectName}.fromJson((e as Map<String, dynamic>))).toList(),\n");
          _des = "\t@JsonKey(name: \'$k\')\n\tList<${objectName}> $k;\n";
          getPropertyFormMapWithCreatClassModel(firstObject, objectName);
        } else {
          _fromJsonDes.write("\t\tsrcJson[\'$k\'] as List,\n");
          _des = "\t@JsonKey(name: \'$k\')\n\tList $k;\n";
        }
      } else if (v is Map) {
        String objectName = "${k[0].toUpperCase()}${k.substring(1)}";
        _fromJsonDes.write(
            "\t\t$objectName.fromJson(srcJson[\'$k\'] as Map) as  ${objectName},\n");
        _des = "\t@JsonKey(name: \'$k\')\n\t${objectName} $k;\n";
        getPropertyFormMapWithCreatClassModel(v, objectName);
      }
      _propertyDes.write(_des + "\n");
      _thisDes.write(_thisKey);
    });

    String header =
        "@JsonSerializable()\nclass ${className} extends ${superClassName} {\n";

    String instanceMethod = "\t$className($_thisDes);\n\n";

    String fromJsonMethod =
        "\tfactory $className.fromJson(Map<String, dynamic> srcJson) => ${className}(\n${_fromJsonDes}\t);\n\n";

    String toJsonMethod =
        "\tMap<String, dynamic> toJson() =>  <String, dynamic>{\n${_toJsonDes}\t};\n\n";

    String footer = "}";

    _classDes.write(header);
    _classDes.write(_propertyDes);
    _classDes.write(instanceMethod);
    _classDes.write(fromJsonMethod);
    _classDes.write(toJsonMethod);
    _classDes.write(footer);

    print(_classDes);

    print("\n//————————————————————————到这里结束————————————————————————");
  }
}
