/*
 * @Author: zlf 
 * @Date: 2020-05-04 16:33:21 
 * @Last Modified by: zlf
 * @Last Modified time: 2020-05-04 17:24:01
 */
import 'dart:convert';

class JsonParsed {
  /**
   * 打印解析后的json (map类型json)
   * 
   * [jsonMap] --->map类型的参数
   * 
   * [level]  --->默认缩进 
   * 
   * 示例:
   *  
   *     JsonParsed.map(jsonMap).printStr();
   * 
   * 查看控制台打印结果
   * */
  static String map(Map jsonMap, [int level = 0]) {
    String tab = getIndentStr(level);
    StringBuffer des = StringBuffer("{\n");
    jsonMap.forEach((k, v) {
      if (v is Map) {
        des.write("$tab\t$k = ");
        String _des = map(v, level + 1);
        des.write("$_des");
      } else if (v is List) {
        String _des = JsonParsed.list(v, level + 1);
        des.write("$tab\t$k = ");
        des.write("$_des,\n");
      } else {
        des.write("$tab\t$k = \"$v\",\n");
      }
    });
    des.write("$tab},\n");
    return des.toString();
  }

  /**
     * 打印解析后的json (map类型json)
     * 
     * [jsonList] --->List类型的参数
     * 
     * [level]  --->默认缩进 
     * 
     * 示例:
     *  
     *     JsonParsed.list(jsonList).printStr();
     * 
     * 查看控制台打印结果
     * */
  static String list(List jsonList, [int level = 0]) {
    String tab = getIndentStr(level);
    StringBuffer des = StringBuffer("[\n");
    jsonList.forEach((o) {
      if (o is Map) {
        String _mapStr = JsonParsed.map(o, level + 1);
        des.write("$tab\t$_mapStr\n");
      } else if (o is List) {
        String d = "$tab\t$o = " + list(o, level + 1);
        des.write(d);
      } else if (o is String) {
        String tab = getIndentStr(level + 1);
        des.write("$tab\"$o\",\n");
      } else {
        des.write("$tab\t$o,\n");
      }
    });
    des.write("$tab]");
    return des.toString();
  }
}

/// 获取缩进
String getIndentStr(int _level) {
  String tab = "";
  if (_level <= 0) {
    return tab;
  }
  for (int i = 0; i < _level; i++) {
    tab = tab + "\t";
  }
  return tab;
}

/// 为String提供额外的打印
extension PrintExtension on String {
  String printStr() {
    print("${this}");
  }
}
